@@ -46,11 +46,12 @@ __FBSDID("$FreeBSD: src/usr.bin/cpio/cmdline.c,v 1.5 2008/12/06 07:30:40 kientzl
 #endif
 
 #include "cpio.h"
+#include "err.h"
 
 /*
  * Short options for cpio.  Please keep this sorted.
  */
-static const char *short_options = "0AaBC:F:O:cdE:f:H:hI:ijLlmnopR:rtuvW:yZz";
+static const char *short_options = "0AaBC:cdE:F:f:H:hI:iJjLlmnO:opR:rtuVvW:yZz";
 
 /*
  * Long options for cpio.  Please keep this sorted.
@@ -61,6 +62,7 @@ static const struct option {
 	int equivalent;	/* Equivalent short option. */
 } cpio_longopts[] = {
 	{ "create",			0, 'o' },
+	{ "dot",			0, 'V' },
 	{ "extract",			0, 'i' },
 	{ "file",			1, 'F' },
 	{ "format",             	1, 'H' },
@@ -68,6 +70,7 @@ static const struct option {
 	{ "insecure",			0, OPTION_INSECURE },
 	{ "link",			0, 'l' },
 	{ "list",			0, 't' },
+	{ "lzma",			0, OPTION_LZMA },
 	{ "make-directories",		0, 'd' },
 	{ "no-preserve-owner",		0, OPTION_NO_PRESERVE_OWNER },
 	{ "null",			0, '0' },
@@ -75,10 +78,12 @@ static const struct option {
 	{ "owner",			1, 'R' },
 	{ "pass-through",		0, 'p' },
 	{ "preserve-modification-time", 0, 'm' },
+	{ "preserve-owner",		0, OPTION_PRESERVE_OWNER },
 	{ "quiet",			0, OPTION_QUIET },
 	{ "unconditional",		0, 'u' },
 	{ "verbose",			0, 'v' },
 	{ "version",			0, OPTION_VERSION },
+	{ "xz",				0, 'J' },
 	{ NULL, 0, 0 }
 };
 
@@ -105,7 +110,7 @@ cpio_getopt(struct cpio *cpio)
 	int opt = '?';
 	int required = 0;
 
-	cpio->optarg = NULL;
+	cpio->argument = NULL;
 
 	/* First time through, initialize everything. */
 	if (state == state_start) {
@@ -171,7 +176,7 @@ cpio_getopt(struct cpio *cpio)
 				/* Otherwise, pick up the next word. */
 				opt_word = *cpio->argv;
 				if (opt_word == NULL) {
-					cpio_warnc(0,
+					lafe_warnc(0,
 					    "Option -%c requires an argument",
 					    opt);
 					return ('?');
@@ -184,7 +189,7 @@ cpio_getopt(struct cpio *cpio)
 				long_prefix = "-W "; /* For clearer errors. */
 			} else {
 				state = state_next_word;
-				cpio->optarg = opt_word;
+				cpio->argument = opt_word;
 			}
 		}
 	}
@@ -198,7 +203,7 @@ cpio_getopt(struct cpio *cpio)
 		p = strchr(opt_word, '=');
 		if (p != NULL) {
 			optlength = (size_t)(p - opt_word);
-			cpio->optarg = (char *)(uintptr_t)(p + 1);
+			cpio->argument = (char *)(uintptr_t)(p + 1);
 		} else {
 			optlength = strlen(opt_word);
 		}
@@ -222,13 +227,13 @@ cpio_getopt(struct cpio *cpio)
 
 		/* Fail if there wasn't a unique match. */
 		if (match == NULL) {
-			cpio_warnc(0,
+			lafe_warnc(0,
 			    "Option %s%s is not supported",
 			    long_prefix, opt_word);
 			return ('?');
 		}
 		if (match2 != NULL) {
-			cpio_warnc(0,
+			lafe_warnc(0,
 			    "Ambiguous option %s%s (matches --%s and --%s)",
 			    long_prefix, opt_word, match->name, match2->name);
 			return ('?');
@@ -237,10 +242,10 @@ cpio_getopt(struct cpio *cpio)
 		/* We've found a unique match; does it need an argument? */
 		if (match->required) {
 			/* Argument required: get next word if necessary. */
-			if (cpio->optarg == NULL) {
-				cpio->optarg = *cpio->argv;
-				if (cpio->optarg == NULL) {
-					cpio_warnc(0,
+			if (cpio->argument == NULL) {
+				cpio->argument = *cpio->argv;
+				if (cpio->argument == NULL) {
+					lafe_warnc(0,
 					    "Option %s%s requires an argument",
 					    long_prefix, match->name);
 					return ('?');
@@ -250,8 +255,8 @@ cpio_getopt(struct cpio *cpio)
 			}
 		} else {
 			/* Argument forbidden: fail if there is one. */
-			if (cpio->optarg != NULL) {
-				cpio_warnc(0,
+			if (cpio->argument != NULL) {
+				lafe_warnc(0,
 				    "Option %s%s does not allow an argument",
 				    long_prefix, match->name);
 				return ('?');
@@ -268,24 +273,37 @@ cpio_getopt(struct cpio *cpio)
  * Parse the argument to the -R or --owner flag.
  *
  * The format is one of the following:
- *   <user>    - Override user but not group
- *   <user>:   - Override both, group is user's default group
- *   <user>:<group> - Override both
- *   :<group>  - Override group but not user
+ *   <username|uid>    - Override user but not group
+ *   <username>:   - Override both, group is user's default group
+ *   <uid>:    - Override user but not group
+ *   <username|uid>:<groupname|gid> - Override both
+ *   :<groupname|gid>  - Override group but not user
+ *
+ * Where uid/gid are decimal representations and groupname/username
+ * are names to be looked up in system database.  Note that we try
+ * to look up an argument as a name first, then try numeric parsing.
  *
  * A period can be used instead of the colon.
  *
- * Sets uid/gid as appropriate, -1 indicates uid/gid not specified.
+ * Sets uid/gid return as appropriate, -1 indicates uid/gid not specified.
+ * TODO: If the spec uses uname/gname, then return those to the caller
+ * as well.  If the spec provides uid/gid, just return names as NULL.
+ *
+ * Returns NULL if no error, otherwise returns error string for display.
  *
  */
-int
+const char *
 owner_parse(const char *spec, int *uid, int *gid)
 {
+	static char errbuff[128];
 	const char *u, *ue, *g;
 
 	*uid = -1;
 	*gid = -1;
 
+	if (spec[0] == '\0')
+		return ("Invalid empty user/group spec");
+
 	/*
 	 * Split spec into [user][:.][group]
 	 *  u -> first char of username, NULL if no username
@@ -312,31 +330,43 @@ owner_parse(const char *spec, int *uid, int *gid)
 		struct passwd *pwent;
 
 		user = (char *)malloc(ue - u + 1);
-		if (user == NULL) {
-			cpio_warnc(errno, "Couldn't allocate memory");
-			return (1);
-		}
+		if (user == NULL)
+			return ("Couldn't allocate memory");
 		memcpy(user, u, ue - u);
 		user[ue - u] = '\0';
-		pwent = getpwnam(user);
-		if (pwent == NULL) {
-			cpio_warnc(errno, "Couldn't lookup user ``%s''", user);
-			return (1);
+		if ((pwent = getpwnam(user)) != NULL) {
+			*uid = pwent->pw_uid;
+			if (*ue != '\0')
+				*gid = pwent->pw_gid;
+		} else {
+			char *end;
+			errno = 0;
+			*uid = (int)strtoul(user, &end, 10);
+			if (errno || *end != '\0') {
+				snprintf(errbuff, sizeof(errbuff),
+				    "Couldn't lookup user ``%s''", user);
+				errbuff[sizeof(errbuff) - 1] = '\0';
+				return (errbuff);
+			}
 		}
 		free(user);
-		*uid = pwent->pw_uid;
-		if (*ue != '\0' && *g == '\0')
-			*gid = pwent->pw_gid;
 	}
+
 	if (*g != '\0') {
 		struct group *grp;
-		grp = getgrnam(g);
-		if (grp != NULL)
+		if ((grp = getgrnam(g)) != NULL) {
 			*gid = grp->gr_gid;
-		else {
-			cpio_warnc(errno, "Couldn't look up group ``%s''", g);
-			return (1);
+		} else {
+			char *end;
+			errno = 0;
+			*gid = (int)strtoul(g, &end, 10);
+			if (errno || *end != '\0') {
+				snprintf(errbuff, sizeof(errbuff),
+				    "Couldn't lookup group ``%s''", g);
+				errbuff[sizeof(errbuff) - 1] = '\0';
+				return (errbuff);
+			}
 		}
 	}
-	return (0);
+	return (NULL);
 }
