@@ -3,8 +3,10 @@ DEFINE_TEST(test_basic)
 DEFINE_TEST(test_cmdline)
 DEFINE_TEST(test_format_newc)
 DEFINE_TEST(test_gcpio_compat)
+DEFINE_TEST(test_option_0)
 DEFINE_TEST(test_option_B_upper)
 DEFINE_TEST(test_option_C_upper)
+DEFINE_TEST(test_option_J_upper)
 DEFINE_TEST(test_option_L_upper)
 DEFINE_TEST(test_option_Z_upper)
 DEFINE_TEST(test_option_a)
@@ -13,6 +15,7 @@ DEFINE_TEST(test_option_d)
 DEFINE_TEST(test_option_f)
 DEFINE_TEST(test_option_help)
 DEFINE_TEST(test_option_l)
+DEFINE_TEST(test_option_lzma)
 DEFINE_TEST(test_option_m)
 DEFINE_TEST(test_option_t)
 DEFINE_TEST(test_option_u)
