@@ -30,11 +30,9 @@ __FBSDID("$FreeBSD: head/lib/libarchive/archive_read_support_format_zip.c 201102
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include <time.h>
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
 #endif
@@ -79,10 +77,10 @@ struct zip {
 	struct zip_entry	*zip_entries;
 	struct zip_entry	*entry;
 
+	size_t			unconsumed;
+
 	/* entry_bytes_remaining is the number of bytes we expect. */
 	int64_t			entry_bytes_remaining;
-	int64_t			entry_offset;
-	size_t			entry_bytes_unconsumed;
 
 	/* These count the number of bytes actually read for the entry. */
 	int64_t			entry_compressed_bytes_read;
@@ -114,6 +112,8 @@ struct zip {
 };
 
 #define ZIP_LENGTH_AT_END	8
+#define ZIP_ENCRYPTED		(1<<0)	
+#define ZIP_STRONG_ENCRYPTED	(1<<6)	
 #define ZIP_UTF8_NAME		(1<<11)	
 
 static int	archive_read_format_zip_streamable_bid(struct archive_read *, int);
@@ -128,9 +128,10 @@ static int	archive_read_format_zip_seekable_read_header(struct archive_read *,
 		    struct archive_entry *);
 static int	archive_read_format_zip_streamable_read_header(struct archive_read *,
 		    struct archive_entry *);
-static int	search_next_signature(struct archive_read *);
+#ifdef HAVE_ZLIB_H
 static int	zip_read_data_deflate(struct archive_read *a, const void **buff,
 		    size_t *size, int64_t *offset);
+#endif
 static int	zip_read_data_none(struct archive_read *a, const void **buff,
 		    size_t *size, int64_t *offset);
 static int	zip_read_local_file_header(struct archive_read *a,
@@ -332,7 +333,6 @@ archive_read_format_zip_seekable_read_header(struct archive_read *a,
 	struct archive_entry *entry)
 {
 	struct zip *zip = (struct zip *)a->format->data;
-	const char *h;
 	int r;
 
 	a->archive.archive_format = ARCHIVE_FORMAT_ZIP;
@@ -353,25 +353,37 @@ archive_read_format_zip_seekable_read_header(struct archive_read *a,
 		return ARCHIVE_EOF;
 	--zip->entries_remaining;
 
-	zip->decompress_init = 0;
-	zip->end_of_entry = 0;
-	zip->entry_uncompressed_bytes_read = 0;
-	zip->entry_compressed_bytes_read = 0;
-	zip->entry_crc32 = crc32(0, NULL, 0);
 	/* TODO: If entries are sorted by offset within the file, we
 	   should be able to skip here instead of seeking.  Skipping is
 	   typically faster (easier for I/O layer to optimize). */
 	__archive_read_seek(a, zip->entry->local_header_offset, SEEK_SET);
-	if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
-		return (ARCHIVE_FATAL);
-
-	if (memcmp(h, "PK\003\004", 4) != 0) {
-		archive_set_error(&a->archive, -1, "Damaged Zip archive");
-		return ARCHIVE_FATAL;
-	}
-
+	zip->unconsumed = 0;
 	r = zip_read_local_file_header(a, entry, zip);
-	return (r);
+	if (r != ARCHIVE_OK)
+		return r;
+	if ((zip->entry->mode & AE_IFMT) == AE_IFLNK) {
+		const void *p;
+		size_t linkname_length = archive_entry_size(entry);
+
+		archive_entry_set_size(entry, 0);
+		p = __archive_read_ahead(a, linkname_length, NULL);
+		if (p == NULL) {
+			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+			    "Truncated Zip file");
+			return ARCHIVE_FATAL;
+		}
+
+		if (archive_entry_copy_symlink_l(entry, p, linkname_length,
+		    NULL) != 0) {
+			/* NOTE: If the last argument is NULL, this will
+			 * fail only by memeory allocation failure. */
+			archive_set_error(&a->archive, ENOMEM,
+			    "Can't allocate memory for Symlink");
+			return (ARCHIVE_FATAL);
+		}
+		/* TODO: handle character-set issues? */
+	}
+	return ARCHIVE_OK;
 }
 
 static int
@@ -438,126 +450,64 @@ static int
 archive_read_format_zip_streamable_read_header(struct archive_read *a,
     struct archive_entry *entry)
 {
-	const void *h;
-	const char *signature;
 	struct zip *zip;
-	int r = ARCHIVE_OK, r1;
 
 	a->archive.archive_format = ARCHIVE_FORMAT_ZIP;
 	if (a->archive.archive_format_name == NULL)
 		a->archive.archive_format_name = "ZIP";
 
 	zip = (struct zip *)(a->format->data);
-	zip->decompress_init = 0;
-	zip->end_of_entry = 0;
-	zip->entry_uncompressed_bytes_read = 0;
-	zip->entry_compressed_bytes_read = 0;
-	zip->entry_crc32 = crc32(0, NULL, 0);
-	if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
-		return (ARCHIVE_FATAL);
 
-	signature = (const char *)h;
-
-	/* If we don't see a PK signature here, scan forward. */
-	if (signature[0] != 'P' || signature[1] != 'K') {
-		r = search_next_signature(a);
-		if (r != ARCHIVE_OK) {
-			archive_set_error(&a->archive,
-			    ARCHIVE_ERRNO_FILE_FORMAT,
-			    "Unable to find next valid PK marker");
-			return (ARCHIVE_FATAL);
-		}
-		if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
-			return (ARCHIVE_FATAL);
-		signature = (const char *)h;
-	}
-
-	/*
-	 * "PK00" signature is used for "split" archives that
-	 * only have a single segment.  This means we can just
-	 * skip the PK00; the first real file header should follow.
-	 */
-	if (signature[2] == '0' && signature[3] == '0') {
-		__archive_read_consume(a, 4);
-		if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
-			return (ARCHIVE_FATAL);
-		signature = (const char *)h;
-		if (signature[0] != 'P' || signature[1] != 'K') {
-			archive_set_error(&a->archive,
-			    ARCHIVE_ERRNO_FILE_FORMAT,
-			    "No valid marker found after PK00");
-			return (ARCHIVE_FATAL);
-		}
-	}
-
-	if (signature[2] == '\001' && signature[3] == '\002') {
-		/* Beginning of central directory. */
-		return (ARCHIVE_EOF);
-	}
-
-	if (signature[2] == '\003' && signature[3] == '\004') {
-		/* Regular file entry. */
+	/* Make sure we have a zip_entry structure to use. */
+	if (zip->zip_entries == NULL) {
+		zip->zip_entries = malloc(sizeof(struct zip_entry));
 		if (zip->zip_entries == NULL) {
-			zip->zip_entries = malloc(sizeof(struct zip_entry));
-			if (zip->zip_entries == NULL) {
-				archive_set_error(&a->archive, ENOMEM, "Out  of memory");
-				return ARCHIVE_FATAL;
-			}
+			archive_set_error(&a->archive, ENOMEM, "Out  of memory");
+			return ARCHIVE_FATAL;
 		}
-		zip->entry = zip->zip_entries;
-		memset(zip->entry, 0, sizeof(struct zip_entry));
-		r1 = zip_read_local_file_header(a, entry, zip);
-		if (r1 != ARCHIVE_OK)
-			return (r1);
-		return (r);
 	}
+	zip->entry = zip->zip_entries;
+	memset(zip->entry, 0, sizeof(struct zip_entry));
 
-	if (signature[2] == '\005' && signature[3] == '\006') {
-		/* End of central directory. */
-		return (ARCHIVE_EOF);
-	}
-
-	archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
-	    "Unable to make sense of PK marker (%d,%d)",
-	    signature[2], signature[3]);
-	return (ARCHIVE_FATAL);
-}
-
-static int
-search_next_signature(struct archive_read *a)
-{
-	const void *h;
-	const char *p, *q;
-	size_t skip;
-	ssize_t bytes;
-	int64_t skipped = 0;
-
+	/* Search ahead for the next local file header. */
+	__archive_read_consume(a, zip->unconsumed);
+	zip->unconsumed = 0;
 	for (;;) {
-		h = __archive_read_ahead(a, 4, &bytes);
-		if (h == NULL)
-			return (ARCHIVE_FATAL);
-		p = h;
-		q = p + bytes;
+		int64_t skipped = 0;
+		const char *p, *end;
+		ssize_t bytes;
 
-		while (p + 4 <= q) {
+		p = __archive_read_ahead(a, 4, &bytes);
+		if (p == NULL)
+			return (ARCHIVE_FATAL);
+		end = p + bytes;
+
+		while (p + 4 <= end) {
 			if (p[0] == 'P' && p[1] == 'K') {
-				if ((p[2] == '\001' && p[3] == '\002')
-				    || (p[2] == '\003' && p[3] == '\004')
-				    || (p[2] == '\005' && p[3] == '\006')
-				    || (p[2] == '0' && p[3] == '0')) {
-					skip = p - (const char *)h;
-					__archive_read_consume(a, skip);
-					return (ARCHIVE_OK);
+				if (p[2] == '\001' && p[3] == '\002')
+					/* Beginning of central directory. */
+					return (ARCHIVE_EOF);
+
+				if (p[2] == '\003' && p[3] == '\004') {
+					/* Regular file entry. */
+					__archive_read_consume(a, skipped);
+					return zip_read_local_file_header(a, entry, zip);
 				}
+
+				if (p[2] == '\005' && p[3] == '\006')
+					/* End of central directory. */
+					return (ARCHIVE_EOF);
 			}
 			++p;
+			++skipped;
 		}
-		skip = p - (const char *)h;
-		__archive_read_consume(a, skip);
-		skipped += skip;
+		__archive_read_consume(a, skipped);
 	}
 }
 
+/*
+ * Assumes file pointer is at beginning of local file header.
+ */
 static int
 zip_read_local_file_header(struct archive_read *a, struct archive_entry *entry,
     struct zip *zip)
@@ -569,11 +519,17 @@ zip_read_local_file_header(struct archive_read *a, struct archive_entry *entry,
 	size_t len, filename_length, extra_length;
 	struct archive_string_conv *sconv;
 	struct zip_entry *zip_entry = zip->entry;
-	uint32_t crc32;
+	uint32_t local_crc32;
 	int64_t compressed_size, uncompressed_size;
 	int ret = ARCHIVE_OK;
 	char version;
 
+	zip->decompress_init = 0;
+	zip->end_of_entry = 0;
+	zip->entry_uncompressed_bytes_read = 0;
+	zip->entry_compressed_bytes_read = 0;
+	zip->entry_crc32 = crc32(0, NULL, 0);
+
 	/* Setup default conversion. */
 	if (zip->sconv == NULL && !zip->init_default_conversion) {
 		zip->sconv_default =
@@ -587,12 +543,16 @@ zip_read_local_file_header(struct archive_read *a, struct archive_entry *entry,
 		return (ARCHIVE_FATAL);
 	}
 
+	if (memcmp(p, "PK\003\004", 4) != 0) {
+		archive_set_error(&a->archive, -1, "Damaged Zip archive");
+		return ARCHIVE_FATAL;
+	}
 	version = p[4];
 	zip_entry->system = p[5];
 	zip_entry->flags = archive_le16dec(p + 6);
 	zip_entry->compression = archive_le16dec(p + 8);
 	zip_entry->mtime = zip_time(p + 10);
-	crc32 = archive_le32dec(p + 14);
+	local_crc32 = archive_le32dec(p + 14);
 	compressed_size = archive_le32dec(p + 18);
 	uncompressed_size = archive_le32dec(p + 22);
 	filename_length = archive_le16dec(p + 26);
@@ -609,7 +569,7 @@ zip_read_local_file_header(struct archive_read *a, struct archive_entry *entry,
 		   which might indicate a damaged file.  But some
 		   writers always put zero in the local header; don't
 		   bother warning about that. */
-		if (crc32 != 0 && crc32 != zip_entry->crc32) {
+		if (local_crc32 != 0 && local_crc32 != zip_entry->crc32) {
 			archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 			    "Inconsistent CRC32 values");
 			ret = ARCHIVE_WARN;
@@ -628,7 +588,7 @@ zip_read_local_file_header(struct archive_read *a, struct archive_entry *entry,
 		}
 	} else {
 		/* If we don't have the CD info, use whatever we do have. */
-		zip_entry->crc32 = crc32;
+		zip_entry->crc32 = local_crc32;
 		zip_entry->compressed_size = compressed_size;
 		zip_entry->uncompressed_size = uncompressed_size;
 	}
@@ -712,7 +672,6 @@ zip_read_local_file_header(struct archive_read *a, struct archive_entry *entry,
 		archive_entry_set_size(entry, zip_entry->uncompressed_size);
 
 	zip->entry_bytes_remaining = zip_entry->compressed_size;
-	zip->entry_offset = 0;
 
 	/* If there's no body, force read_data() to return EOF immediately. */
 	if (0 == (zip_entry->flags & ZIP_LENGTH_AT_END)
@@ -775,60 +734,47 @@ static int
 archive_read_format_zip_read_data(struct archive_read *a,
     const void **buff, size_t *size, int64_t *offset)
 {
-	int r, reset_buff = 0;
-	struct zip *zip;
+	int r;
+	struct zip *zip = (struct zip *)(a->format->data);
 
-	zip = (struct zip *)(a->format->data);
+	*offset = zip->entry_uncompressed_bytes_read;
+	*size = 0;
+	*buff = NULL;
 
-	if (zip->entry_bytes_unconsumed) {
-		__archive_read_consume(a, zip->entry_bytes_unconsumed);
-		zip->entry_bytes_unconsumed = 0;
-	}
-
-	/*
-	 * If we hit end-of-entry last time, clean up and return
-	 * ARCHIVE_EOF this time.
-	 */
-	if (zip->end_of_entry) {
-		*offset = zip->entry_uncompressed_bytes_read;
-		*size = 0;
-		*buff = NULL;
+	/* If we hit end-of-entry last time, return ARCHIVE_EOF. */
+	if (zip->end_of_entry)
 		return (ARCHIVE_EOF);
+
+	/* Return EOF immediately if this is a non-regular file. */
+	if (AE_IFREG != (zip->entry->mode & AE_IFMT))
+		return (ARCHIVE_EOF);
+
+	if (zip->entry->flags & (ZIP_ENCRYPTED | ZIP_STRONG_ENCRYPTED)) {
+		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
+		    "Encrypted file is unsupported");
+		return (ARCHIVE_FAILED);
 	}
 
+	__archive_read_consume(a, zip->unconsumed);
+	zip->unconsumed = 0;
+
 	switch(zip->entry->compression) {
 	case 0:  /* No compression. */
 		r =  zip_read_data_none(a, buff, size, offset);
-		reset_buff = 1;
 		break;
+#ifdef HAVE_ZLIB_H
 	case 8: /* Deflate compression. */
 		r =  zip_read_data_deflate(a, buff, size, offset);
-		if (zip->entry_bytes_unconsumed) {
-			__archive_read_consume(a, zip->entry_bytes_unconsumed);
-			zip->entry_bytes_unconsumed = 0;
-		}
 		break;
+#endif
 	default: /* Unsupported compression. */
-		*buff = NULL;
-		*size = 0;
-		*offset = 0;
 		/* Return a warning. */
 		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Unsupported ZIP compression method (%s)",
 		    compression_name(zip->entry->compression));
-		if (zip->entry->flags & ZIP_LENGTH_AT_END) {
-			/*
-			 * ZIP_LENGTH_AT_END requires us to
-			 * decompress the entry in order to
-			 * skip it, but we don't know this
-			 * compression method, so we give up.
-			 */
-			r = ARCHIVE_FATAL;
-		} else {
-			/* We can't decompress this entry, but we will
-			 * be able to skip() it and try the next entry. */
-			r = ARCHIVE_WARN;
-		}
+		/* We can't decompress this entry, but we will
+		 * be able to skip() it and try the next entry. */
+		return (ARCHIVE_FAILED);
 		break;
 	}
 	if (r != ARCHIVE_OK)
@@ -838,40 +784,6 @@ archive_read_format_zip_read_data(struct archive_read *a,
 		zip->entry_crc32 = crc32(zip->entry_crc32, *buff, *size);
 	/* If we hit the end, swallow any end-of-data marker. */
 	if (zip->end_of_entry) {
-		if (zip->entry->flags & ZIP_LENGTH_AT_END) {
-			const char *p;
-
-			/* since we're screwing with the exposed window,
-			 * it's possible this will cross a forced move/realloc
-			 * w/in that layer... in other words, the previous
-			 * read_ahead's returned pointer isn't trustable.
-			 * thus we redo the window, and reset buff if needed.
-			 * note that decompression sidesteps this via the
-			 * flushing of entry_bytes_unconsumed
-			 */
-			if (NULL == (p = __archive_read_ahead(a,
-			    zip->entry_bytes_unconsumed + 16, NULL))) {
-				archive_set_error(&a->archive,
-				    ARCHIVE_ERRNO_FILE_FORMAT,
-				    "Truncated ZIP end-of-file record");
-				return (ARCHIVE_FATAL);
-			}
-			/* update the ptr, and set p to just the 16 bytes
-			 * we care about */
-			if (reset_buff) {
-				*buff = p;
-				p += zip->entry_bytes_unconsumed;
-			}
-			/* Consume the optional PK\007\010 marker. */
-			if (p[0] == 'P' && p[1] == 'K' && p[2] == '\007' && p[3] == '\010') {
-				zip->entry_bytes_unconsumed += 4;
-				p += 4;
-			}
-			zip->entry->crc32 = archive_le32dec(p);
-			zip->entry->compressed_size = archive_le32dec(p + 4);
-			zip->entry->uncompressed_size = archive_le32dec(p + 8);
-			zip->entry_bytes_unconsumed += 12;
-		}
 		/* Check file size, CRC against these values. */
 		if (zip->entry->compressed_size != zip->entry_compressed_bytes_read) {
 			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
@@ -900,9 +812,6 @@ archive_read_format_zip_read_data(struct archive_read *a,
 		}
 	}
 
-	/* Return EOF immediately if this is a non-regular file. */
-	if (AE_IFREG != (zip->entry->mode & AE_IFMT))
-		return (ARCHIVE_EOF);
 	return (ARCHIVE_OK);
 }
 
@@ -938,9 +847,6 @@ zip_read_data_none(struct archive_read *a, const void **_buff,
 	ssize_t bytes_avail;
 
 	zip = (struct zip *)(a->format->data);
-	*_buff = NULL;
-	*size = 0;
-	*offset = zip->entry_offset;
 
 	if (zip->entry->flags & ZIP_LENGTH_AT_END) {
 		const char *p;
@@ -963,7 +869,11 @@ zip_read_data_none(struct archive_read *a, const void **_buff,
 		    && archive_le32dec(p + 4) == zip->entry_crc32
 		    && archive_le32dec(p + 8) == zip->entry_compressed_bytes_read
 		    && archive_le32dec(p + 12) == zip->entry_uncompressed_bytes_read) {
+			zip->entry->crc32 = archive_le32dec(p + 4);
+			zip->entry->compressed_size = archive_le32dec(p + 8);
+			zip->entry->uncompressed_size = archive_le32dec(p + 12);
 			zip->end_of_entry = 1;
+			zip->unconsumed = 16;
 			return (ARCHIVE_OK);
 		}
 		/* If not at EOF, ensure we consume at least one byte. */
@@ -998,11 +908,10 @@ zip_read_data_none(struct archive_read *a, const void **_buff,
 			bytes_avail = zip->entry_bytes_remaining;
 	}
 	*size = bytes_avail;
-	zip->entry_offset += bytes_avail;
 	zip->entry_bytes_remaining -= bytes_avail;
 	zip->entry_uncompressed_bytes_read += bytes_avail;
 	zip->entry_compressed_bytes_read += bytes_avail;
-	zip->entry_bytes_unconsumed = bytes_avail;
+	zip->unconsumed += bytes_avail;
 	*_buff = buff;
 	return (ARCHIVE_OK);
 }
@@ -1021,7 +930,7 @@ zip_read_data_deflate(struct archive_read *a, const void **buff,
 
 	/* If the buffer hasn't been allocated, allocate it now. */
 	if (zip->uncompressed_buffer == NULL) {
-		zip->uncompressed_buffer_size = 32 * 1024;
+		zip->uncompressed_buffer_size = 256 * 1024;
 		zip->uncompressed_buffer
 		    = (unsigned char *)malloc(zip->uncompressed_buffer_size);
 		if (zip->uncompressed_buffer == NULL) {
@@ -1098,81 +1007,106 @@ zip_read_data_deflate(struct archive_read *a, const void **buff,
 
 	/* Consume as much as the compressor actually used. */
 	bytes_avail = zip->stream.total_in;
-	zip->entry_bytes_unconsumed = bytes_avail;
+	__archive_read_consume(a, bytes_avail);
 	zip->entry_bytes_remaining -= bytes_avail;
 	zip->entry_compressed_bytes_read += bytes_avail;
 
-	*offset = zip->entry_offset;
 	*size = zip->stream.total_out;
-	zip->entry_uncompressed_bytes_read += *size;
+	zip->entry_uncompressed_bytes_read += zip->stream.total_out;
 	*buff = zip->uncompressed_buffer;
-	zip->entry_offset += *size;
+
+	if (zip->end_of_entry && (zip->entry->flags & ZIP_LENGTH_AT_END)) {
+		const char *p;
+
+		if (NULL == (p = __archive_read_ahead(a, 16, NULL))) {
+			archive_set_error(&a->archive,
+			    ARCHIVE_ERRNO_FILE_FORMAT,
+			    "Truncated ZIP end-of-file record");
+			return (ARCHIVE_FATAL);
+		}
+		/* Consume the optional PK\007\010 marker. */
+		if (p[0] == 'P' && p[1] == 'K' && p[2] == '\007' && p[3] == '\010') {
+			zip->entry->crc32 = archive_le32dec(p + 4);
+			zip->entry->compressed_size = archive_le32dec(p + 8);
+			zip->entry->uncompressed_size = archive_le32dec(p + 12);
+			zip->unconsumed = 16;
+		}
+	}
+
 	return (ARCHIVE_OK);
 }
-#else
-static int
-zip_read_data_deflate(struct archive_read *a, const void **buff,
-    size_t *size, int64_t *offset)
-{
-	*buff = NULL;
-	*size = 0;
-	*offset = 0;
-	archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
-	    "libarchive compiled without deflate support (no libz)");
-	return (ARCHIVE_FATAL);
-}
 #endif
 
 static int
 archive_read_format_zip_read_data_skip(struct archive_read *a)
 {
 	struct zip *zip;
-	int64_t bytes_skipped;
 
 	zip = (struct zip *)(a->format->data);
 
-	#define flush_unconsumed()	\
-	if (zip->entry_bytes_unconsumed) {							\
-		__archive_read_consume(a, zip->entry_bytes_unconsumed);	\
-		zip->entry_bytes_unconsumed = 0;						\
-	}
-
-	flush_unconsumed();
-
 	/* If we've already read to end of data, we're done. */
 	if (zip->end_of_entry)
 		return (ARCHIVE_OK);
+	/* If we're seeking, we're done. */
+	if (zip->have_central_directory)
+		return (ARCHIVE_OK);
 
-	/*
-	 * If the length is at the end, we have no choice but
-	 * to decompress all the data to find the end marker.
-	 */
-	if (zip->entry->flags & ZIP_LENGTH_AT_END) {
-		size_t size;
-		int64_t offset;
-		int r;
-		do {
-			const void *buff = NULL;
-			r = archive_read_format_zip_read_data(a, &buff,
-			    &size, &offset);
-			flush_unconsumed();
-		} while (r == ARCHIVE_OK);
-		return (r);
+	/* So we know we're streaming... */
+	if (0 == (zip->entry->flags & ZIP_LENGTH_AT_END)) {
+		/* We know the compressed length, so we can just skip. */
+		int64_t bytes_skipped = __archive_read_consume(a,
+		    zip->entry_bytes_remaining + zip->unconsumed);
+		if (bytes_skipped < 0)
+			return (ARCHIVE_FATAL);
+		zip->unconsumed = 0;
+		return (ARCHIVE_OK);
 	}
 
-	#undef flush_consumed
-
-	/*
-	 * If the length is at the beginning, we can skip the
-	 * compressed data much more quickly.
-	 */
-	bytes_skipped = __archive_read_consume(a, zip->entry_bytes_remaining);
-	if (bytes_skipped < 0)
-		return (ARCHIVE_FATAL);
-
-	/* This entry is finished and done. */
-	zip->end_of_entry = 1;
-	return (ARCHIVE_OK);
+	/* We're streaming and we don't know the length. */
+	/* If the body is compressed and we know the format, we can
+	 * find an exact end-of-entry by decompressing it. */
+	switch (zip->entry->compression) {
+#ifdef HAVE_ZLIB_H
+	case 8: /* Deflate compression. */
+		while (!zip->end_of_entry) {
+			int64_t offset = 0;
+			const void *buff = NULL;
+			size_t size = 0;
+			int r;
+			r =  zip_read_data_deflate(a, &buff, &size, &offset);
+			if (r != ARCHIVE_OK)
+				return (r);
+		}
+		break;
+#endif
+	default: /* Uncompressed or unknown. */
+		/* Scan for a PK\007\010 signature. */
+		__archive_read_consume(a, zip->unconsumed);
+		zip->unconsumed = 0;
+		for (;;) {
+			const char *p, *buff;
+			ssize_t bytes_avail;
+			buff = __archive_read_ahead(a, 16, &bytes_avail);
+			if (bytes_avail < 16) {
+				archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
+				    "Truncated ZIP file data");
+				return (ARCHIVE_FATAL);
+			}
+			p = buff;
+			while (p < buff + bytes_avail - 16) {
+				if (p[3] == 'P') { p += 3; }
+				else if (p[3] == 'K') { p += 2; }
+				else if (p[3] == '\007') { p += 1; }
+				else if (p[3] == '\010' && p[2] == '\007'
+				    && p[1] == 'K' && p[0] == 'P') {
+					__archive_read_consume(a, p - buff + 16);
+					return ARCHIVE_OK;
+				} else { p += 4; }
+			}
+			__archive_read_consume(a, p - buff);
+		}
+	}
+	return ARCHIVE_OK;
 }
 
 static int
