@@ -542,11 +542,36 @@ la_CreateHardLinkW(wchar_t *linkname, wchar_t *target)
 {
 	static BOOLEAN (WINAPI *f)(LPWSTR, LPWSTR, LPSECURITY_ATTRIBUTES);
 	static int set;
+	BOOL ret;
+
 	if (!set) {
 		set = 1;
 		f = la_GetFunctionKernel32("CreateHardLinkW");
 	}
-	return f == NULL ? 0 : (*f)(linkname, target, NULL);
+	if (!f)
+		return (0);
+	ret = (*f)(linkname, target, NULL);
+	if (!ret) {
+		/* Under windows 2000, it is necessary to remove
+		 * the "\\?\" prefix. */
+#define IS_UNC(name)	((name[0] == L'U' || name[0] == L'u') &&	\
+			 (name[1] == L'N' || name[1] == L'n') &&	\
+			 (name[2] == L'C' || name[2] == L'c') &&	\
+			 name[3] == L'\\')
+		if (!wcsncmp(linkname,L"\\\\?\\", 4)) {
+			linkname += 4;
+			if (IS_UNC(linkname))
+				linkname += 4;
+		}
+		if (!wcsncmp(target,L"\\\\?\\", 4)) {
+			target += 4;
+			if (IS_UNC(target))
+				target += 4;
+		}
+#undef IS_UNC
+		ret = (*f)(linkname, target, NULL);
+	}
+	return (ret);
 }
 
 static int
@@ -585,7 +610,7 @@ lazy_stat(struct archive_write_disk *a)
 
 	/*
 	 * XXX At this point, symlinks should not be hit, otherwise
-	 * XXX a race occured.  Do we want to check explicitly for that?
+	 * XXX a race occurred.  Do we want to check explicitly for that?
 	 */
 	if (file_information(a, a->name, &a->st, NULL, 1) == 0) {
 		a->pst = &a->st;
@@ -1037,48 +1062,11 @@ _archive_write_disk_finish_entry(struct archive *_a)
 		/* Last write ended at exactly the filesize; we're done. */
 		/* Hopefully, this is the common case. */
 	} else {
-		if (la_ftruncate(a->fh, a->filesize) == -1 &&
-		    a->filesize == 0) {
+		if (la_ftruncate(a->fh, a->filesize) == -1) {
 			archive_set_error(&a->archive, errno,
 			    "File size could not be restored");
 			return (ARCHIVE_FAILED);
 		}
-		/*
-		 * Not all platforms implement the XSI option to
-		 * extend files via ftruncate.  Stat() the file again
-		 * to see what happened.
-		 */
-		a->pst = NULL;
-		if ((ret = lazy_stat(a)) != ARCHIVE_OK)
-			return (ret);
-		/* We can use lseek()/write() to extend the file if
-		 * ftruncate didn't work or isn't available. */
-		if (bhfi_size(&(a->st)) < a->filesize) {
-			const char nul = '\0';
-			LARGE_INTEGER distance;
-			distance.QuadPart = a->filesize - 1;
-			if (!SetFilePointerEx_perso(a->fh, distance, NULL, FILE_BEGIN)) {
-				DWORD lasterr = GetLastError();
-				if (lasterr == ERROR_ACCESS_DENIED)
-					errno = EBADF;
-				else
-					la_dosmaperr(lasterr);
-				archive_set_error(&a->archive, errno,
-				    "Seek failed");
-				return (ARCHIVE_FATAL);
-			}
-			if (!WriteFile(a->fh, &nul, 1, NULL, NULL)) {
-				DWORD lasterr = GetLastError();
-				if (lasterr == ERROR_ACCESS_DENIED)
-					errno = EBADF;
-				else
-					la_dosmaperr(lasterr);
-				archive_set_error(&a->archive, errno,
-				    "Write to restore size failed");
-				return (ARCHIVE_FATAL);
-			}
-			a->pst = NULL;
-		}
 	}
 
 	/* Restore metadata. */
@@ -1495,7 +1483,7 @@ create_filesystem_object(struct archive_write_disk *a)
 		 *
 		 * If the hardlink was successfully created and
 		 * the archive doesn't have carry data for it,
-		 * consider it to be non-authoritive for meta data.
+		 * consider it to be non-authoritative for meta data.
 		 * This is consistent with GNU tar and BSD pax.
 		 * If the hardlink does carry data, let the last
 		 * archive entry decide ownership.
@@ -2207,6 +2195,7 @@ create_dir(struct archive_write_disk *a, wchar_t *path)
 			le->fixup |=TODO_MODE_BASE;
 			le->mode = mode_final;
 		}
+		free(full);
 		return (ARCHIVE_OK);
 	} else {
 		la_dosmaperr(GetLastError());
