@@ -24,7 +24,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_write_open_filename.c,v 1.19 2007/01/09 08:05:56 kientzle Exp $");
+__FBSDID("$FreeBSD: head/lib/libarchive/archive_write_open_filename.c 191165 2009-04-17 00:39:35Z kientzle $");
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
@@ -46,10 +46,19 @@ __FBSDID("$FreeBSD: src/lib/libarchive/archive_write_open_filename.c,v 1.19 2007
 #endif
 
 #include "archive.h"
+#include "archive_string.h"
+
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
 
 struct write_file_data {
 	int		fd;
-	char		filename[1];
+	char		mbs_filename;
+	union {
+		char		m[1];
+		wchar_t		w[1];
+	} filename; /* Must be last! */
 };
 
 static int	file_close(struct archive *, void *);
@@ -67,26 +76,68 @@ archive_write_open_filename(struct archive *a, const char *filename)
 {
 	struct write_file_data *mine;
 
-	if (filename == NULL || filename[0] == '\0') {
-		mine = (struct write_file_data *)malloc(sizeof(*mine));
-		if (mine == NULL) {
-			archive_set_error(a, ENOMEM, "No memory");
-			return (ARCHIVE_FATAL);
-		}
-		mine->filename[0] = '\0'; /* Record that we're using stdout. */
-	} else {
-		mine = (struct write_file_data *)malloc(sizeof(*mine) + strlen(filename));
-		if (mine == NULL) {
-			archive_set_error(a, ENOMEM, "No memory");
-			return (ARCHIVE_FATAL);
-		}
-		strcpy(mine->filename, filename);
+	if (filename == NULL || filename[0] == '\0')
+		return (archive_write_open_fd(a, 1));
+
+	mine = (struct write_file_data *)malloc(sizeof(*mine) + strlen(filename));
+	if (mine == NULL) {
+		archive_set_error(a, ENOMEM, "No memory");
+		return (ARCHIVE_FATAL);
 	}
+	strcpy(mine->filename.m, filename);
+	mine->mbs_filename = 1;
 	mine->fd = -1;
 	return (archive_write_open(a, mine,
-		    file_open, file_write, file_close));
+		file_open, file_write, file_close));
 }
 
+int
+archive_write_open_filename_w(struct archive *a, const wchar_t *filename)
+{
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	struct write_file_data *mine;
+
+	if (filename == NULL || filename[0] == L'\0')
+		return (archive_write_open_fd(a, 1));
+
+	mine = malloc(sizeof(*mine) + wcslen(filename) * sizeof(wchar_t));
+	if (mine == NULL) {
+		archive_set_error(a, ENOMEM, "No memory");
+		return (ARCHIVE_FATAL);
+	}
+	wcscpy(mine->filename.w, filename);
+	mine->mbs_filename = 0;
+	mine->fd = -1;
+	return (archive_write_open(a, mine,
+		file_open, file_write, file_close));
+#else
+	/*
+	 * POSIX system does not support a wchar_t interface for
+	 * open() system call, so we have to translate a wchar_t
+	 * filename to multi-byte one and use it.
+	 */
+	struct archive_string fn;
+	int r;
+
+	if (filename == NULL || filename[0] == L'\0')
+		return (archive_write_open_fd(a, 1));
+
+	archive_string_init(&fn);
+	if (archive_string_append_from_wcs(&fn, filename,
+	    wcslen(filename)) != 0) {
+		archive_set_error(a, EINVAL,
+		    "Failed to convert a wide-character filename to"
+		    " a multi-byte filename");
+		archive_string_free(&fn);
+		return (ARCHIVE_FATAL);
+	}
+	r = archive_write_open_filename(a, fn.s);
+	archive_string_free(&fn);
+	return (r);
+#endif
+}
+
+
 static int
 file_open(struct archive *a, void *client_data)
 {
@@ -95,32 +146,51 @@ file_open(struct archive *a, void *client_data)
 	struct stat st;
 
 	mine = (struct write_file_data *)client_data;
-	flags = O_WRONLY | O_CREAT | O_TRUNC;
+	flags = O_WRONLY | O_CREAT | O_TRUNC | O_BINARY;
 
 	/*
 	 * Open the file.
 	 */
-	if (mine->filename[0] != '\0') {
-		mine->fd = open(mine->filename, flags, 0666);
+	if (mine->mbs_filename) {
+		mine->fd = open(mine->filename.m, flags, 0666);
 		if (mine->fd < 0) {
 			archive_set_error(a, errno, "Failed to open '%s'",
-			    mine->filename);
+			    mine->filename.m);
+			return (ARCHIVE_FATAL);
+		}
+
+		if (fstat(mine->fd, &st) != 0) {
+			archive_set_error(a, errno, "Couldn't stat '%s'",
+			    mine->filename.m);
 			return (ARCHIVE_FATAL);
 		}
 	} else {
-		/*
-		 * NULL filename is stdout.
-		 */
-		mine->fd = 1;
-		/* By default, pad archive when writing to stdout. */
-		if (archive_write_get_bytes_in_last_block(a) < 0)
-			archive_write_set_bytes_in_last_block(a, 0);
-	}
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		mine->fd = _wopen(mine->filename.w, flags, 0666);
+		if (mine->fd < 0 && errno == ENOENT) {
+			wchar_t *fullpath;
+			fullpath = __la_win_permissive_name_w(mine->filename.w);
+			if (fullpath != NULL) {
+				mine->fd = _wopen(fullpath, flags, 0666);
+				free(fullpath);
+			}
+		}
+		if (mine->fd < 0) {
+			archive_set_error(a, errno, "Failed to open '%S'",
+			    mine->filename.w);
+			return (ARCHIVE_FATAL);
+		}
 
-	if (fstat(mine->fd, &st) != 0) {
-               archive_set_error(a, errno, "Couldn't stat '%s'",
-                   mine->filename);
-               return (ARCHIVE_FATAL);
+		if (fstat(mine->fd, &st) != 0) {
+			archive_set_error(a, errno, "Couldn't stat '%S'",
+			    mine->filename.w);
+			return (ARCHIVE_FATAL);
+		}
+#else
+		archive_set_error(a, ARCHIVE_ERRNO_MISC,
+		    "Unexpedted operation in archive_write_open_filename");
+		return (ARCHIVE_FATAL);
+#endif
 	}
 
 	/*
@@ -154,12 +224,16 @@ file_write(struct archive *a, void *client_data, const void *buff, size_t length
 	ssize_t	bytesWritten;
 
 	mine = (struct write_file_data *)client_data;
-	bytesWritten = write(mine->fd, buff, length);
-	if (bytesWritten <= 0) {
-		archive_set_error(a, errno, "Write error");
-		return (-1);
+	for (;;) {
+		bytesWritten = write(mine->fd, buff, length);
+		if (bytesWritten <= 0) {
+			if (errno == EINTR)
+				continue;
+			archive_set_error(a, errno, "Write error");
+			return (-1);
+		}
+		return (bytesWritten);
 	}
-	return (bytesWritten);
 }
 
 static int
@@ -168,8 +242,7 @@ file_close(struct archive *a, void *client_data)
 	struct write_file_data	*mine = (struct write_file_data *)client_data;
 
 	(void)a; /* UNUSED */
-	if (mine->filename[0] != '\0')
-		close(mine->fd);
+	close(mine->fd);
 	free(mine);
 	return (ARCHIVE_OK);
 }
