@@ -46,6 +46,10 @@ __FBSDID("$FreeBSD$");
 #include <zlib.h>
 #endif
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 #include "archive.h"
 #include "archive_crypto_private.h"
 #include "archive_endian.h"
@@ -684,21 +688,24 @@ xar_write_data(struct archive_write *a, const void *buff, size_t s)
 
 		archive_string_empty(&(xar->cur_file->script));
 		if (rsize > 2 && b[0] == '#' && b[1] == '!') {
-			char path[PATH_MAX];
 			size_t i, end, off;
 
-			end = sizeof(path);
-			if (end > rsize)
-				end = rsize;
 			off = 2;
 			if (b[off] == ' ')
 				off++;
+#ifdef PATH_MAX
+			if ((rsize - off) > PATH_MAX)
+				end = off + PATH_MAX;
+			else
+#endif
+				end = rsize;
+			/* Find the end of a script path. */
 			for (i = off; i < end && b[i] != '\0' &&
 			    b[i] != '\n' && b[i] != '\r' &&
 			    b[i] != ' ' && b[i] != '\t'; i++)
-				path[i - off] = b[i];
-			path[i - off] = '\0';
-			archive_strcpy(&(xar->cur_file->script), path);
+				;
+			archive_strncpy(&(xar->cur_file->script), b + off,
+			    i - off);
 		}
 	}
 #endif
