@@ -23,27 +23,21 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_write_compress_program.c,v 1.3 2008/09/01 05:38:33 kientzle Exp $");
+__FBSDID("$FreeBSD: head/lib/libarchive/test/test_write_compress_program.c 201247 2009-12-30 05:59:21Z kientzle $");
 
 char buff[1000000];
 char buff2[64];
 
 DEFINE_TEST(test_write_compress_program)
 {
-#if ARCHIVE_VERSION_NUMBER < 1009000
-	skipping("archive_write_set_compress_program()");
-#else
 	struct archive_entry *ae;
 	struct archive *a;
 	size_t used;
 	int blocksize = 1024;
 	int r;
-	const char *compprog, *decompprog;
 
-	decompprog = external_gzip_program(1);
-	if ((compprog = external_gzip_program(0)) == NULL) {
-		skipping("There is no gzip compression "
-		    "program in this platform");
+	if (!canGzip()) {
+		skipping("Cannot run 'gzip'");
 		return;
 	}
 
@@ -51,11 +45,11 @@ DEFINE_TEST(test_write_compress_program)
 	/* Write it through an external "gzip" program. */
 	assert((a = archive_write_new()) != NULL);
 	assertA(0 == archive_write_set_format_ustar(a));
-	r = archive_write_set_compression_program(a, compprog);
+	r = archive_write_set_compression_program(a, "gzip");
 	if (r == ARCHIVE_FATAL) {
 		skipping("Write compression via external "
 		    "program unsupported on this platform");
-		archive_write_finish(a);
+		archive_write_free(a);
 		return;
 	}
 	assertA(0 == archive_write_set_bytes_per_block(a, blocksize));
@@ -78,29 +72,29 @@ DEFINE_TEST(test_write_compress_program)
 	assertA(8 == archive_write_data(a, "12345678", 9));
 
 	/* Close out the archive. */
-	assertA(0 == archive_write_close(a));
-	assertA(0 == archive_write_finish(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_write_close(a));
+	assertEqualInt(ARCHIVE_OK, archive_write_free(a));
 
 	/*
 	 * Now, read the data back through the built-in gzip support.
 	 */
 	assert((a = archive_read_new()) != NULL);
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
-	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_compression_all(a));
-	r = archive_read_support_compression_gzip(a);
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_filter_all(a));
+	r = archive_read_support_filter_gzip(a);
 	/* The compression_gzip() handler will fall back to gunzip
 	 * automatically, but if we know gunzip isn't available, then
 	 * skip the rest. */
-	if (r != ARCHIVE_OK && decompprog == NULL) {
-		skipping("No gzip decompression is available; "
+	if (r != ARCHIVE_OK && !canGunzip()) {
+		skipping("No libz and no gunzip program, "
 		    "unable to verify gzip compression");
-		assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
+		assertEqualInt(ARCHIVE_OK, archive_read_free(a));
 		return;
 	}
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_open_memory(a, buff, used));
 
 	if (!assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae))) {
-		archive_read_finish(a);
+		archive_read_free(a);
 		return;
 	}
 
@@ -116,6 +110,5 @@ DEFINE_TEST(test_write_compress_program)
 	/* Verify the end of the archive. */
 	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
-	assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
-#endif
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
 }
